@@ -53,15 +53,14 @@
 extern struct isis *isis;
 extern struct thread_master *master;
 
-struct isis_nexthop *
+static struct isis_nexthop *
 isis_nexthop_create (struct in_addr *ip, unsigned int ifindex)
 {
   struct listnode *node;
   struct isis_nexthop *nexthop;
 
-  for (node = listhead (isis->nexthops); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (isis->nexthops, node, nexthop))
     {
-      nexthop = getdata (node);
       if (nexthop->ifindex != ifindex)
 	continue;
       if (ip && memcmp (&nexthop->ip, ip, sizeof (struct in_addr)) != 0)
@@ -86,7 +85,7 @@ isis_nexthop_create (struct in_addr *ip, unsigned int ifindex)
   return nexthop;
 }
 
-void
+static void
 isis_nexthop_delete (struct isis_nexthop *nexthop)
 {
   nexthop->lock--;
@@ -99,16 +98,15 @@ isis_nexthop_delete (struct isis_nexthop *nexthop)
   return;
 }
 
-int
+static int
 nexthoplookup (struct list *nexthops, struct in_addr *ip,
 	       unsigned int ifindex)
 {
   struct listnode *node;
   struct isis_nexthop *nh;
 
-  for (node = listhead (nexthops); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (nexthops, node, nh))
     {
-      nh = getdata (node);
       if (!(memcmp (ip, &nh->ip, sizeof (struct in_addr))) &&
 	  ifindex == nh->ifindex)
 	return 1;
@@ -117,7 +115,8 @@ nexthoplookup (struct list *nexthops, struct in_addr *ip,
   return 0;
 }
 
-void
+#if 0 /* Old or new code? */
+static void
 nexthop_print (struct isis_nexthop *nh)
 {
   u_char buf[BUFSIZ];
@@ -127,18 +126,18 @@ nexthop_print (struct isis_nexthop *nh)
   zlog_debug ("      %s %u", buf, nh->ifindex);
 }
 
-void
+static void
 nexthops_print (struct list *nhs)
 {
   struct listnode *node;
 
-  for (node = listhead (nhs); node; nextnode (node))
-    nexthop_print (getdata (node));
+  for (ALL_LIST_ELEMENTS_RO (nhs, node, nh))
+    nexthop_print (nh);
 }
+#endif /* 0 */
 
 #ifdef HAVE_IPV6
-
-struct isis_nexthop6 *
+static struct isis_nexthop6 *
 isis_nexthop6_new (struct in6_addr *ip6, unsigned int ifindex)
 {
 
@@ -158,15 +157,14 @@ isis_nexthop6_new (struct in6_addr *ip6, unsigned int ifindex)
   return nexthop6;
 }
 
-struct isis_nexthop6 *
+static struct isis_nexthop6 *
 isis_nexthop6_create (struct in6_addr *ip6, unsigned int ifindex)
 {
   struct listnode *node;
   struct isis_nexthop6 *nexthop6;
 
-  for (node = listhead (isis->nexthops6); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (isis->nexthops6, node, nexthop6))
     {
-      nexthop6 = getdata (node);
       if (nexthop6->ifindex != ifindex)
 	continue;
       if (ip6 && memcmp (&nexthop6->ip6, ip6, sizeof (struct in6_addr)) != 0)
@@ -181,7 +179,7 @@ isis_nexthop6_create (struct in6_addr *ip6, unsigned int ifindex)
   return nexthop6;
 }
 
-void
+static void
 isis_nexthop6_delete (struct isis_nexthop6 *nexthop6)
 {
 
@@ -195,16 +193,15 @@ isis_nexthop6_delete (struct isis_nexthop6 *nexthop6)
   return;
 }
 
-int
+static int
 nexthop6lookup (struct list *nexthops6, struct in6_addr *ip6,
 		unsigned int ifindex)
 {
   struct listnode *node;
   struct isis_nexthop6 *nh6;
 
-  for (node = listhead (nexthops6); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (nexthops6, node, nh6))
     {
-      nh6 = getdata (node);
       if (!(memcmp (ip6, &nh6->ip6, sizeof (struct in6_addr))) &&
 	  ifindex == nh6->ifindex)
 	return 1;
@@ -213,7 +210,8 @@ nexthop6lookup (struct list *nexthops6, struct in6_addr *ip6,
   return 0;
 }
 
-void
+#ifdef EXTREME_DEBUG
+static void
 nexthop6_print (struct isis_nexthop6 *nh6)
 {
   u_char buf[BUFSIZ];
@@ -223,30 +221,29 @@ nexthop6_print (struct isis_nexthop6 *nh6)
   zlog_debug ("      %s %u", buf, nh6->ifindex);
 }
 
-void
+static void
 nexthops6_print (struct list *nhs6)
 {
   struct listnode *node;
 
-  for (node = listhead (nhs6); node; nextnode (node))
-    nexthop6_print (getdata (node));
+  for (ALL_LIST_ELEMENTS_RO (nhs6, node, nh6))
+    nexthop6_print (nh6);
 }
-
-
+#endif /* EXTREME_DEBUG */
 #endif /* HAVE_IPV6 */
 
-void
+static void
 adjinfo2nexthop (struct list *nexthops, struct isis_adjacency *adj)
 {
   struct isis_nexthop *nh;
-  struct listnode *node;
+  struct listnode *node, *nnode;
   struct in_addr *ipv4_addr;
 
   if (adj->ipv4_addrs == NULL)
     return;
-  for (node = listhead (adj->ipv4_addrs); node; nextnode (node))
+
+  for (ALL_LIST_ELEMENTS (adj->ipv4_addrs, node, nnode, ipv4_addr))
     {
-      ipv4_addr = getdata (node);
       if (!nexthoplookup (nexthops, ipv4_addr,
 			  adj->circuit->interface->ifindex))
 	{
@@ -258,19 +255,18 @@ adjinfo2nexthop (struct list *nexthops, struct isis_adjacency *adj)
 }
 
 #ifdef HAVE_IPV6
-void
+static void
 adjinfo2nexthop6 (struct list *nexthops6, struct isis_adjacency *adj)
 {
-  struct listnode *node;
+  struct listnode *node, *nnode;
   struct in6_addr *ipv6_addr;
   struct isis_nexthop6 *nh6;
 
   if (!adj->ipv6_addrs)
     return;
 
-  for (node = listhead (adj->ipv6_addrs); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS (adj->ipv6_addrs, node, nnode, ipv6_addr))
     {
-      ipv6_addr = getdata (node);
       if (!nexthop6lookup (nexthops6, ipv6_addr,
 			   adj->circuit->interface->ifindex))
 	{
@@ -282,13 +278,13 @@ adjinfo2nexthop6 (struct list *nexthops6, struct isis_adjacency *adj)
 }
 #endif /* HAVE_IPV6 */
 
-struct isis_route_info *
+static struct isis_route_info *
 isis_route_info_new (uint32_t cost, uint32_t depth, u_char family,
 		     struct list *adjacencies)
 {
   struct isis_route_info *rinfo;
   struct isis_adjacency *adj;
-  struct listnode *node;
+  struct listnode *node, *nnode;
 
   rinfo = XMALLOC (MTYPE_ISIS_ROUTE_INFO, sizeof (struct isis_route_info));
   if (!rinfo)
@@ -301,21 +297,15 @@ isis_route_info_new (uint32_t cost, uint32_t depth, u_char family,
   if (family == AF_INET)
     {
       rinfo->nexthops = list_new ();
-      for (node = listhead (adjacencies); node; nextnode (node))
-	{
-	  adj = getdata (node);
-	  adjinfo2nexthop (rinfo->nexthops, adj);
-	}
+      for (ALL_LIST_ELEMENTS (adjacencies, node, nnode, adj))
+        adjinfo2nexthop (rinfo->nexthops, adj);
     }
 #ifdef HAVE_IPV6
   if (family == AF_INET6)
     {
       rinfo->nexthops6 = list_new ();
-      for (node = listhead (adjacencies); node; nextnode (node))
-	{
-	  adj = getdata (node);
-	  adjinfo2nexthop6 (rinfo->nexthops6, adj);
-	}
+      for (ALL_LIST_ELEMENTS (adjacencies, node, nnode, adj))
+        adjinfo2nexthop6 (rinfo->nexthops6, adj);
     }
 
 #endif /* HAVE_IPV6 */
@@ -326,7 +316,7 @@ isis_route_info_new (uint32_t cost, uint32_t depth, u_char family,
   return rinfo;
 }
 
-void
+static void
 isis_route_info_delete (struct isis_route_info *route_info)
 {
   if (route_info->nexthops)
@@ -346,7 +336,7 @@ isis_route_info_delete (struct isis_route_info *route_info)
   XFREE (MTYPE_ISIS_ROUTE_INFO, route_info);
 }
 
-int
+static int
 isis_route_info_same_attrib (struct isis_route_info *new,
 			     struct isis_route_info *old)
 {
@@ -358,11 +348,11 @@ isis_route_info_same_attrib (struct isis_route_info *new,
   return 1;
 }
 
-int
+static int
 isis_route_info_same (struct isis_route_info *new,
 		      struct isis_route_info *old, u_char family)
 {
-  struct listnode *node;
+  struct listnode *node, *nnode;
   struct isis_nexthop *nexthop;
 #ifdef HAVE_IPV6
   struct isis_nexthop6 *nexthop6;
@@ -372,55 +362,42 @@ isis_route_info_same (struct isis_route_info *new,
 
   if (family == AF_INET)
     {
-      for (node = listhead (new->nexthops); node; nextnode (node))
-	{
-	  nexthop = (struct isis_nexthop *) getdata (node);
-	  if (nexthoplookup (old->nexthops, &nexthop->ip, nexthop->ifindex) ==
-	      0)
-	    return 0;
-	}
+      for (ALL_LIST_ELEMENTS (new->nexthops, node, nnode, nexthop))
+        if (nexthoplookup (old->nexthops, &nexthop->ip, nexthop->ifindex) 
+              == 0)
+          return 0;
 
-      for (node = listhead (old->nexthops); node; nextnode (node))
-	{
-	  nexthop = (struct isis_nexthop *) getdata (node);
-	  if (nexthoplookup (new->nexthops, &nexthop->ip, nexthop->ifindex) ==
-	      0)
-	    return 0;
-	}
+      for (ALL_LIST_ELEMENTS (old->nexthops, node, nnode, nexthop))
+        if (nexthoplookup (new->nexthops, &nexthop->ip, nexthop->ifindex) 
+             == 0)
+          return 0;
     }
 #ifdef HAVE_IPV6
   else if (family == AF_INET6)
     {
-      for (node = listhead (new->nexthops6); node; nextnode (node))
-	{
-	  nexthop6 = (struct isis_nexthop6 *) getdata (node);
-	  if (nexthop6lookup (old->nexthops6, &nexthop6->ip6,
-			      nexthop6->ifindex) == 0)
-	    return 0;
-	}
+      for (ALL_LIST_ELEMENTS (new->nexthops6, node, nnode, nexthop6))
+        if (nexthop6lookup (old->nexthops6, &nexthop6->ip6,
+                            nexthop6->ifindex) == 0)
+          return 0;
 
-      for (node = listhead (old->nexthops6); node; nextnode (node))
-	{
-	  nexthop6 = (struct isis_nexthop6 *) getdata (node);
-	  if (nexthop6lookup (new->nexthops6, &nexthop6->ip6,
-			      nexthop6->ifindex) == 0)
-	    return 0;
-	}
+      for (ALL_LIST_ELEMENTS (old->nexthops6, node, nnode, nexthop6))
+        if (nexthop6lookup (new->nexthops6, &nexthop6->ip6,
+                            nexthop6->ifindex) == 0)
+          return 0;
     }
 #endif /* HAVE_IPV6 */
 
   return 1;
 }
 
-void
+static void
 isis_nexthops_merge (struct list *new, struct list *old)
 {
-  struct listnode *node;
+  struct listnode *node, *nnode;
   struct isis_nexthop *nexthop;
 
-  for (node = listhead (new); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS (new, node, nnode, nexthop))
     {
-      nexthop = (struct isis_nexthop *) getdata (node);
       if (nexthoplookup (old, &nexthop->ip, nexthop->ifindex))
 	continue;
       listnode_add (old, nexthop);
@@ -429,15 +406,14 @@ isis_nexthops_merge (struct list *new, struct list *old)
 }
 
 #ifdef HAVE_IPV6
-void
+static void
 isis_nexthops6_merge (struct list *new, struct list *old)
 {
-  struct listnode *node;
+  struct listnode *node, *nnode;
   struct isis_nexthop6 *nexthop6;
 
-  for (node = listhead (new); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS (new, node, nnode, nexthop6))
     {
-      nexthop6 = (struct isis_nexthop6 *) getdata (node);
       if (nexthop6lookup (old, &nexthop6->ip6, nexthop6->ifindex))
 	continue;
       listnode_add (old, nexthop6);
@@ -446,7 +422,7 @@ isis_nexthops6_merge (struct list *new, struct list *old)
 }
 #endif /* HAVE_IPV6 */
 
-void
+static void
 isis_route_info_merge (struct isis_route_info *new,
 		       struct isis_route_info *old, u_char family)
 {
@@ -460,7 +436,7 @@ isis_route_info_merge (struct isis_route_info *new,
   return;
 }
 
-int
+static int
 isis_route_info_prefer_new (struct isis_route_info *new,
 			    struct isis_route_info *old)
 {
@@ -565,7 +541,7 @@ isis_route_create (struct prefix *prefix, u_int32_t cost, u_int32_t depth,
   return route_info;
 }
 
-void
+static void
 isis_route_delete (struct prefix *prefix, struct route_table *table)
 {
   struct route_node *rode;
