@@ -1,6 +1,6 @@
 /*
-   $Id: command.c,v 1.31.2.3 2005/03/07 13:39:20 hasso Exp $
-
+   $Id: command.c,v 1.47 2005/04/25 16:26:42 paul Exp $
+ 
    Command interpreter routine for virtual terminal [aka TeletYpe]
    Copyright (C) 1997, 98, 99 Kunihiro Ishiguro
 
@@ -31,6 +31,7 @@ Boston, MA 02111-1307, USA.  */
 #include "vector.h"
 #include "vty.h"
 #include "command.h"
+#include "workqueue.h"
 
 /* Command vector which includes some level of command lists. Normally
    each daemon maintains each own cmdvec. */
@@ -157,31 +158,24 @@ char *
 argv_concat (const char **argv, int argc, int shift)
 {
   int i;
-  int len;
-  int index;
+  size_t len;
   char *str;
+  char *p;
 
-  str = NULL;
-  index = 0;
-
+  len = 0;
+  for (i = shift; i < argc; i++)
+    len += strlen(argv[i])+1;
+  if (!len)
+    return NULL;
+  p = str = XMALLOC(MTYPE_TMP, len);
   for (i = shift; i < argc; i++)
     {
-      len = strlen (argv[i]);
-
-      if (i == shift)
-	{
-	  str = XSTRDUP (MTYPE_TMP, argv[i]);
-	  index = len;
-	}
-      else
-	{
-	  str = XREALLOC (MTYPE_TMP, str, (index + len + 2));
-	  str[index++] = ' ';
-	  memcpy (str + index, argv[i], len);
-	  index += len;
-	  str[index] = '\0';
-	}
+      size_t arglen;
+      memcpy(p, argv[i], (arglen = strlen(argv[i])));
+      p += arglen;
+      *p++ = ' ';
     }
+  *(p-1) = '\0';
   return str;
 }
 
@@ -223,18 +217,21 @@ sort_node ()
   vector descvec;
   struct cmd_element *cmd_element;
 
-  for (i = 0; i < vector_max (cmdvec); i++) 
+  for (i = 0; i < vector_active (cmdvec); i++) 
     if ((cnode = vector_slot (cmdvec, i)) != NULL)
       {	
 	vector cmd_vector = cnode->cmd_vector;
-	qsort (cmd_vector->index, cmd_vector->max, sizeof (void *), cmp_node);
+	qsort (cmd_vector->index, vector_active (cmd_vector), 
+	       sizeof (void *), cmp_node);
 
-	for (j = 0; j < vector_max (cmd_vector); j++)
-	  if ((cmd_element = vector_slot (cmd_vector, j)) != NULL)
+	for (j = 0; j < vector_active (cmd_vector); j++)
+	  if ((cmd_element = vector_slot (cmd_vector, j)) != NULL
+	      && vector_active (cmd_element->strvec))
 	    {
 	      descvec = vector_slot (cmd_element->strvec,
-				     vector_max (cmd_element->strvec) - 1);
-	      qsort (descvec->index, descvec->max, sizeof (void *), cmp_desc);
+				     vector_active (cmd_element->strvec) - 1);
+	      qsort (descvec->index, vector_active (descvec), 
+	             sizeof (void *), cmp_desc);
 	    }
       }
 }
@@ -301,7 +298,7 @@ cmd_free_strvec (vector v)
   if (!v)
     return;
 
-  for (i = 0; i < vector_max (v); i++)
+  for (i = 0; i < vector_active (v); i++)
     if ((cp = vector_slot (v, i)) != NULL)
       XFREE (MTYPE_STRVEC, cp);
 
@@ -444,15 +441,14 @@ cmd_cmdsize (vector strvec)
   unsigned int i;
   int size = 0;
   vector descvec;
+  struct desc *desc;
 
-  for (i = 0; i < vector_max (strvec); i++)
+  for (i = 0; i < vector_active (strvec); i++)
+    if ((descvec = vector_slot (strvec, i)) != NULL)
     {
-      descvec = vector_slot (strvec, i);
-
-      if (vector_max (descvec) == 1)
+      if ((vector_active (descvec)) == 1
+        && (desc = vector_slot (descvec, 0)) != NULL)
 	{
-	  struct desc *desc = vector_slot (descvec, 0);
-
 	  if (desc->cmd == NULL || CMD_OPTION (desc->cmd))
 	    return size;
 	  else
@@ -604,7 +600,9 @@ config_write_host (struct vty *vty)
     vty_out (vty, "service terminal-length %d%s", host.lines,
 	     VTY_NEWLINE);
 
-  if (! host.motd)
+  if (host.motdfile)
+    vty_out (vty, "banner motd file %s%s", host.motdfile, VTY_NEWLINE);
+  else if (! host.motd)
     vty_out (vty, "no banner motd%s", VTY_NEWLINE);
 
   return 1;
@@ -1125,14 +1123,14 @@ cmd_filter_by_completion (char *command, vector v, unsigned int index)
   enum match_type match_type;
   vector descvec;
   struct desc *desc;
-  
+
   match_type = no_match;
 
   /* If command and cmd_element string does not match set NULL to vector */
-  for (i = 0; i < vector_max (v); i++) 
+  for (i = 0; i < vector_active (v); i++)
     if ((cmd_element = vector_slot (v, i)) != NULL)
       {
-	if (index >= vector_max (cmd_element->strvec))
+	if (index >= vector_active (cmd_element->strvec))
 	  vector_slot (v, i) = NULL;
 	else
 	  {
@@ -1140,90 +1138,90 @@ cmd_filter_by_completion (char *command, vector v, unsigned int index)
 	    int matched = 0;
 
 	    descvec = vector_slot (cmd_element->strvec, index);
-	    
-	    for (j = 0; j < vector_max (descvec); j++)
-	      {
-		desc = vector_slot (descvec, j);
-		str = desc->cmd;
 
-		if (CMD_VARARG (str))
-		  {
-		    if (match_type < vararg_match)
-		      match_type = vararg_match;
-		    matched++;
-		  }
-		else if (CMD_RANGE (str))
-		  {
-		    if (cmd_range_match (str, command))
-		      {
-			if (match_type < range_match)
-			  match_type = range_match;
+	    for (j = 0; j < vector_active (descvec); j++)
+	      if ((desc = vector_slot (descvec, j)))
+		{
+		  str = desc->cmd;
+		  
+		  if (CMD_VARARG (str))
+		    {
+		      if (match_type < vararg_match)
+			match_type = vararg_match;
+		      matched++;
+		    }
+		  else if (CMD_RANGE (str))
+		    {
+		      if (cmd_range_match (str, command))
+			{
+			  if (match_type < range_match)
+			    match_type = range_match;
 
-			matched++;
-		      }
-		  }
+			  matched++;
+			}
+		    }
 #ifdef HAVE_IPV6
-		else if (CMD_IPV6 (str))
-		  {
-		    if (cmd_ipv6_match (command))
-		      {
-			if (match_type < ipv6_match)
-			  match_type = ipv6_match;
+		  else if (CMD_IPV6 (str))
+		    {
+		      if (cmd_ipv6_match (command))
+			{
+			  if (match_type < ipv6_match)
+			    match_type = ipv6_match;
 
-			matched++;
-		      }
-		  }
-		else if (CMD_IPV6_PREFIX (str))
-		  {
-		    if (cmd_ipv6_prefix_match (command))
-		      {
-			if (match_type < ipv6_prefix_match)
-			  match_type = ipv6_prefix_match;
+			  matched++;
+			}
+		    }
+		  else if (CMD_IPV6_PREFIX (str))
+		    {
+		      if (cmd_ipv6_prefix_match (command))
+			{
+			  if (match_type < ipv6_prefix_match)
+			    match_type = ipv6_prefix_match;
 
-			matched++;
-		      }
-		  }
-#endif   /* HAVE_IPV6  */
-		else if (CMD_IPV4 (str))
-		  {
-		    if (cmd_ipv4_match (command))
-		      {
-			if (match_type < ipv4_match)
-			  match_type = ipv4_match;
+			  matched++;
+			}
+		    }
+#endif /* HAVE_IPV6  */
+		  else if (CMD_IPV4 (str))
+		    {
+		      if (cmd_ipv4_match (command))
+			{
+			  if (match_type < ipv4_match)
+			    match_type = ipv4_match;
 
-			matched++;
-		      }
-		  }
-		else if (CMD_IPV4_PREFIX (str))
-		  {
-		    if (cmd_ipv4_prefix_match (command))
-		      {
-			if (match_type < ipv4_prefix_match)
-			  match_type = ipv4_prefix_match;
-			matched++;
-		      }
-		  }
-		else
-		/* Check is this point's argument optional ? */
-		if (CMD_OPTION (str) || CMD_VARIABLE (str))
-		  {
-		    if (match_type < extend_match)
-		      match_type = extend_match;
-		    matched++;
-		  }
-		else if (strncmp (command, str, strlen (command)) == 0)
-		  {
-		    if (strcmp (command, str) == 0) 
-		      match_type = exact_match;
-		    else
-		      {
-			if (match_type < partly_match)
-			  match_type = partly_match;
-		      }
-		    matched++;
-		  }
-	      }
-	    if (! matched)
+			  matched++;
+			}
+		    }
+		  else if (CMD_IPV4_PREFIX (str))
+		    {
+		      if (cmd_ipv4_prefix_match (command))
+			{
+			  if (match_type < ipv4_prefix_match)
+			    match_type = ipv4_prefix_match;
+			  matched++;
+			}
+		    }
+		  else
+		    /* Check is this point's argument optional ? */
+		  if (CMD_OPTION (str) || CMD_VARIABLE (str))
+		    {
+		      if (match_type < extend_match)
+			match_type = extend_match;
+		      matched++;
+		    }
+		  else if (strncmp (command, str, strlen (command)) == 0)
+		    {
+		      if (strcmp (command, str) == 0)
+			match_type = exact_match;
+		      else
+			{
+			  if (match_type < partly_match)
+			    match_type = partly_match;
+			}
+		      matched++;
+		    }
+		}
+	    if (!matched)
 	      vector_slot (v, i) = NULL;
 	  }
       }
@@ -1240,98 +1238,98 @@ cmd_filter_by_string (char *command, vector v, unsigned int index)
   enum match_type match_type;
   vector descvec;
   struct desc *desc;
-  
+
   match_type = no_match;
 
   /* If command and cmd_element string does not match set NULL to vector */
-  for (i = 0; i < vector_max (v); i++) 
+  for (i = 0; i < vector_active (v); i++)
     if ((cmd_element = vector_slot (v, i)) != NULL)
       {
 	/* If given index is bigger than max string vector of command,
-           set NULL*/
-	if (index >= vector_max (cmd_element->strvec))
+	   set NULL */
+	if (index >= vector_active (cmd_element->strvec))
 	  vector_slot (v, i) = NULL;
-	else 
+	else
 	  {
 	    unsigned int j;
 	    int matched = 0;
 
 	    descvec = vector_slot (cmd_element->strvec, index);
 
-	    for (j = 0; j < vector_max (descvec); j++)
-	      {
-		desc = vector_slot (descvec, j);
-		str = desc->cmd;
+	    for (j = 0; j < vector_active (descvec); j++)
+	      if ((desc = vector_slot (descvec, j)))
+		{
+		  str = desc->cmd;
 
-		if (CMD_VARARG (str))
-		  {
-		    if (match_type < vararg_match)
-		      match_type = vararg_match;
-		    matched++;
-		  }
-		else if (CMD_RANGE (str))
-		  {
-		    if (cmd_range_match (str, command))
-		      {
-			if (match_type < range_match)
-			  match_type = range_match;
-			matched++;
-		      }
-		  }
+		  if (CMD_VARARG (str))
+		    {
+		      if (match_type < vararg_match)
+			match_type = vararg_match;
+		      matched++;
+		    }
+		  else if (CMD_RANGE (str))
+		    {
+		      if (cmd_range_match (str, command))
+			{
+			  if (match_type < range_match)
+			    match_type = range_match;
+			  matched++;
+			}
+		    }
 #ifdef HAVE_IPV6
-		else if (CMD_IPV6 (str))
-		  {
-		    if (cmd_ipv6_match (command) == exact_match)
-		      {
-			if (match_type < ipv6_match)
-			  match_type = ipv6_match;
-			matched++;
-		      }
-		  }
-		else if (CMD_IPV6_PREFIX (str))
-		  {
-		    if (cmd_ipv6_prefix_match (command) == exact_match)
-		      {
-			if (match_type < ipv6_prefix_match)
-			  match_type = ipv6_prefix_match;
-			matched++;
-		      }
-		  }
+		  else if (CMD_IPV6 (str))
+		    {
+		      if (cmd_ipv6_match (command) == exact_match)
+			{
+			  if (match_type < ipv6_match)
+			    match_type = ipv6_match;
+			  matched++;
+			}
+		    }
+		  else if (CMD_IPV6_PREFIX (str))
+		    {
+		      if (cmd_ipv6_prefix_match (command) == exact_match)
+			{
+			  if (match_type < ipv6_prefix_match)
+			    match_type = ipv6_prefix_match;
+			  matched++;
+			}
+		    }
 #endif /* HAVE_IPV6  */
-		else if (CMD_IPV4 (str))
-		  {
-		    if (cmd_ipv4_match (command) == exact_match)
-		      {
-			if (match_type < ipv4_match)
-			  match_type = ipv4_match;
-			matched++;
-		      }
-		  }
-		else if (CMD_IPV4_PREFIX (str))
-		  {
-		    if (cmd_ipv4_prefix_match (command) == exact_match)
-		      {
-			if (match_type < ipv4_prefix_match)
-			  match_type = ipv4_prefix_match;
-			matched++;
-		      }
-		  }
-		else if (CMD_OPTION (str) || CMD_VARIABLE (str))
-		  {
-		    if (match_type < extend_match)
-		      match_type = extend_match;
-		    matched++;
-		  }
-		else
-		  {		  
-		    if (strcmp (command, str) == 0)
-		      {
-			match_type = exact_match;
-			matched++;
-		      }
-		  }
-	      }
-	    if (! matched)
+		  else if (CMD_IPV4 (str))
+		    {
+		      if (cmd_ipv4_match (command) == exact_match)
+			{
+			  if (match_type < ipv4_match)
+			    match_type = ipv4_match;
+			  matched++;
+			}
+		    }
+		  else if (CMD_IPV4_PREFIX (str))
+		    {
+		      if (cmd_ipv4_prefix_match (command) == exact_match)
+			{
+			  if (match_type < ipv4_prefix_match)
+			    match_type = ipv4_prefix_match;
+			  matched++;
+			}
+		    }
+		  else if (CMD_OPTION (str) || CMD_VARIABLE (str))
+		    {
+		      if (match_type < extend_match)
+			match_type = extend_match;
+		      matched++;
+		    }
+		  else
+		    {
+		      if (strcmp (command, str) == 0)
+			{
+			  match_type = exact_match;
+			  matched++;
+			}
+		    }
+		}
+	    if (!matched)
 	      vector_slot (v, i) = NULL;
 	  }
       }
@@ -1349,87 +1347,87 @@ is_cmd_ambiguous (char *command, vector v, int index, enum match_type type)
   const char *matched = NULL;
   vector descvec;
   struct desc *desc;
-  
-  for (i = 0; i < vector_max (v); i++) 
+
+  for (i = 0; i < vector_active (v); i++)
     if ((cmd_element = vector_slot (v, i)) != NULL)
       {
 	int match = 0;
 
 	descvec = vector_slot (cmd_element->strvec, index);
 
-	for (j = 0; j < vector_max (descvec); j++)
-	  {
-	    enum match_type ret;
+	for (j = 0; j < vector_active (descvec); j++)
+	  if ((desc = vector_slot (descvec, j)))
+	    {
+	      enum match_type ret;
+	      
+	      str = desc->cmd;
 
-	    desc = vector_slot (descvec, j);
-	    str = desc->cmd;
+	      switch (type)
+		{
+		case exact_match:
+		  if (!(CMD_OPTION (str) || CMD_VARIABLE (str))
+		      && strcmp (command, str) == 0)
+		    match++;
+		  break;
+		case partly_match:
+		  if (!(CMD_OPTION (str) || CMD_VARIABLE (str))
+		      && strncmp (command, str, strlen (command)) == 0)
+		    {
+		      if (matched && strcmp (matched, str) != 0)
+			return 1;	/* There is ambiguous match. */
+		      else
+			matched = str;
+		      match++;
+		    }
+		  break;
+		case range_match:
+		  if (cmd_range_match (str, command))
+		    {
+		      if (matched && strcmp (matched, str) != 0)
+			return 1;
+		      else
+			matched = str;
+		      match++;
+		    }
+		  break;
+#ifdef HAVE_IPV6
+		case ipv6_match:
+		  if (CMD_IPV6 (str))
+		    match++;
+		  break;
+		case ipv6_prefix_match:
+		  if ((ret = cmd_ipv6_prefix_match (command)) != no_match)
+		    {
+		      if (ret == partly_match)
+			return 2;	/* There is incomplete match. */
 
-	    switch (type)
-	      {
-	      case exact_match:
-		if (! (CMD_OPTION (str) || CMD_VARIABLE (str))
-		    && strcmp (command, str) == 0)
-		  match++;
-		break;
-	      case partly_match:
-		if (! (CMD_OPTION (str) || CMD_VARIABLE (str))
-		    && strncmp (command, str, strlen (command)) == 0)
-		  {
-		    if (matched && strcmp (matched, str) != 0)
-		      return 1;	/* There is ambiguous match. */
-		    else
-		      matched = str;
+		      match++;
+		    }
+		  break;
+#endif /* HAVE_IPV6 */
+		case ipv4_match:
+		  if (CMD_IPV4 (str))
 		    match++;
-		  }
-		break;
-	      case range_match:
-		if (cmd_range_match (str, command))
-		  {
-		    if (matched && strcmp (matched, str) != 0)
-		      return 1;
-		    else
-		      matched = str;
-		    match++;
-		  }
-		break;
-#ifdef HAVE_IPV6		
- 	      case ipv6_match:
-		if (CMD_IPV6 (str))
-		  match++;
-		break;
-	      case ipv6_prefix_match:
-		if ((ret = cmd_ipv6_prefix_match (command)) != no_match)
-		  {
-		    if (ret == partly_match)
-		      return 2; /* There is incomplete match. */
+		  break;
+		case ipv4_prefix_match:
+		  if ((ret = cmd_ipv4_prefix_match (command)) != no_match)
+		    {
+		      if (ret == partly_match)
+			return 2;	/* There is incomplete match. */
 
+		      match++;
+		    }
+		  break;
+		case extend_match:
+		  if (CMD_OPTION (str) || CMD_VARIABLE (str))
 		    match++;
-		  }
-		break;
-#endif  /* HAVE_IPV6 */		
-	      case ipv4_match:
-		if (CMD_IPV4 (str))
-		  match++;
-		break;
-	      case ipv4_prefix_match:
-		if ((ret = cmd_ipv4_prefix_match (command)) != no_match)
-		  {
-		    if (ret == partly_match)
-		      return 2; /* There is incomplete match. */
-
-		    match++;
-		  }
-		break;
-	      case extend_match:
-		if (CMD_OPTION (str) || CMD_VARIABLE (str))
-		  match++;
-		break;
-	      case no_match:
-	      default:
-		break;
-	      }
-	  }
-	if (! match)
+		  break;
+		case no_match:
+		default:
+		  break;
+		}
+	    }
+	if (!match)
 	  vector_slot (v, i) = NULL;
       }
   return 0;
@@ -1528,7 +1526,7 @@ cmd_unique_string (vector v, const char *str)
   unsigned int i;
   char *match;
 
-  for (i = 0; i < vector_max (v); i++)
+  for (i = 0; i < vector_active (v); i++)
     if ((match = vector_slot (v, i)) != NULL)
       if (strcmp (match, str) == 0)
 	return 0;
@@ -1543,7 +1541,7 @@ desc_unique_string (vector v, const char *str)
   unsigned int i;
   struct desc *desc;
 
-  for (i = 0; i < vector_max (v); i++)
+  for (i = 0; i < vector_active (v); i++)
     if ((desc = vector_slot (v, i)) != NULL)
       if (strcmp (desc->cmd, str) == 0)
 	return 1;
@@ -1566,20 +1564,26 @@ cmd_try_do_shortcut (enum node_type node, char* first_word) {
 static vector
 cmd_describe_command_real (vector vline, struct vty *vty, int *status)
 {
-  int i;
+  unsigned int i;
   vector cmd_vector;
 #define INIT_MATCHVEC_SIZE 10
   vector matchvec;
   struct cmd_element *cmd_element;
-  int index;
+  unsigned int index;
   int ret;
   enum match_type match;
   char *command;
   static struct desc desc_cr = { "<cr>", "" };
 
   /* Set index. */
-  index = vector_max (vline) - 1;
-
+  if (vector_active (vline) == 0)
+    {
+      *status = CMD_ERR_NO_MATCH;
+      return NULL;
+    }
+  else
+    index = vector_active (vline) - 1;
+  
   /* Make copy vector of current node's command vector. */
   cmd_vector = vector_copy (cmd_node_vector (cmdvec, vty->node));
 
@@ -1589,47 +1593,48 @@ cmd_describe_command_real (vector vline, struct vty *vty, int *status)
   /* Filter commands. */
   /* Only words precedes current word will be checked in this loop. */
   for (i = 0; i < index; i++)
-    {
-      command = vector_slot (vline, i);
-      match = cmd_filter_by_completion (command, cmd_vector, i);
+    if ((command = vector_slot (vline, i)))
+      {
+	match = cmd_filter_by_completion (command, cmd_vector, i);
+	
+	if (match == vararg_match)
+	  {
+	    struct cmd_element *cmd_element;
+	    vector descvec;
+	    unsigned int j, k;
 
-      if (match == vararg_match)
-	{
-	  struct cmd_element *cmd_element;
-	  vector descvec;
-	  unsigned int j, k;
+	    for (j = 0; j < vector_active (cmd_vector); j++)
+	      if ((cmd_element = vector_slot (cmd_vector, j)) != NULL
+		  && (vector_active (cmd_element->strvec)))
+		{
+		  descvec = vector_slot (cmd_element->strvec,
+					 vector_active (cmd_element->strvec) - 1);
+		  for (k = 0; k < vector_active (descvec); k++)
+		    {
+		      struct desc *desc = vector_slot (descvec, k);
+		      vector_set (matchvec, desc);
+		    }
+		}
+            
+	    vector_set (matchvec, &desc_cr);
+	    vector_free (cmd_vector);
 
-	  for (j = 0; j < vector_max (cmd_vector); j++)
-	    if ((cmd_element = vector_slot (cmd_vector, j)) != NULL)
-	      {
-		descvec = vector_slot (cmd_element->strvec,
-				       vector_max (cmd_element->strvec) - 1);
-		for (k = 0; k < vector_max (descvec); k++)
-		  {
-		    struct desc *desc = vector_slot (descvec, k);
-		    vector_set (matchvec, desc);
-		  }
-	      }
+	    return matchvec;
+	  }
 
-	  vector_set (matchvec, &desc_cr);
-	  vector_free (cmd_vector);
-
-	  return matchvec;
-	}
-
-      if ((ret = is_cmd_ambiguous (command, cmd_vector, i, match)) == 1)
-	{
-	  vector_free (cmd_vector);
-	  *status = CMD_ERR_AMBIGUOUS;
-	  return NULL;
-	}
-      else if (ret == 2)
-	{
-	  vector_free (cmd_vector);
-	  *status = CMD_ERR_NO_MATCH;
-	  return NULL;
-	}
-    }
+	if ((ret = is_cmd_ambiguous (command, cmd_vector, i, match)) == 1)
+	  {
+	    vector_free (cmd_vector);
+	    *status = CMD_ERR_AMBIGUOUS;
+	    return NULL;
+	  }
+	else if (ret == 2)
+	  {
+	    vector_free (cmd_vector);
+	    *status = CMD_ERR_NO_MATCH;
+	    return NULL;
+	  }
+      }
 
   /* Prepare match vector */
   /*  matchvec = vector_init (INIT_MATCHVEC_SIZE); */
@@ -1640,22 +1645,22 @@ cmd_describe_command_real (vector vline, struct vty *vty, int *status)
     match = cmd_filter_by_completion (command, cmd_vector, index);
 
   /* Make description vector. */
-  for (i = 0; i < vector_max (cmd_vector); i++)
+  for (i = 0; i < vector_active (cmd_vector); i++)
     if ((cmd_element = vector_slot (cmd_vector, i)) != NULL)
       {
 	const char *string = NULL;
 	vector strvec = cmd_element->strvec;
 
-        /* if command is NULL, index may be equal to vector_max */
-	if (command && index >= vector_max (strvec))
+	/* if command is NULL, index may be equal to vector_active */
+	if (command && index >= vector_active (strvec))
 	  vector_slot (cmd_vector, i) = NULL;
 	else
 	  {
 	    /* Check if command is completed. */
-            if (command == NULL && index == vector_max (strvec))
+	    if (command == NULL && index == vector_active (strvec))
 	      {
 		string = "<cr>";
-		if (! desc_unique_string (matchvec, string))
+		if (!desc_unique_string (matchvec, string))
 		  vector_set (matchvec, &desc_cr);
 	      }
 	    else
@@ -1664,17 +1669,17 @@ cmd_describe_command_real (vector vline, struct vty *vty, int *status)
 		vector descvec = vector_slot (strvec, index);
 		struct desc *desc;
 
-		for (j = 0; j < vector_max (descvec); j++)
-		  {
-		    desc = vector_slot (descvec, j);
-		    string = cmd_entry_function_desc (command, desc->cmd);
-		    if (string)
-		      {
-			/* Uniqueness check */
-			if (! desc_unique_string (matchvec, string))
-			  vector_set (matchvec, desc);
-		      }
-		  }
+		for (j = 0; j < vector_active (descvec); j++)
+		  if ((desc = vector_slot (descvec, j)))
+		    {
+		      string = cmd_entry_function_desc (command, desc->cmd);
+		      if (string)
+			{
+			  /* Uniqueness check */
+			  if (!desc_unique_string (matchvec, string))
+			    vector_set (matchvec, desc);
+			}
+		    }
 	      }
 	  }
       }
@@ -1683,7 +1688,7 @@ cmd_describe_command_real (vector vline, struct vty *vty, int *status)
   if (vector_slot (matchvec, 0) == NULL)
     {
       vector_free (matchvec);
-      *status= CMD_ERR_NO_MATCH;
+      *status = CMD_ERR_NO_MATCH;
     }
   else
     *status = CMD_SUCCESS;
@@ -1708,7 +1713,7 @@ cmd_describe_command (vector vline, struct vty *vty, int *status)
 
       shifted_vline = vector_init (vector_count(vline));
       /* use memcpy? */
-      for (index = 1; index < vector_max (vline); index++) 
+      for (index = 1; index < vector_active (vline); index++) 
 	{
 	  vector_set_index (shifted_vline, index-1, vector_lookup(vline, index));
 	}
@@ -1762,74 +1767,81 @@ cmd_lcd (char **matched)
 static char **
 cmd_complete_command_real (vector vline, struct vty *vty, int *status)
 {
-  int i;
+  unsigned int i;
   vector cmd_vector = vector_copy (cmd_node_vector (cmdvec, vty->node));
 #define INIT_MATCHVEC_SIZE 10
   vector matchvec;
   struct cmd_element *cmd_element;
-  int index = vector_max (vline) - 1;
+  unsigned int index;
   char **match_str;
   struct desc *desc;
   vector descvec;
   char *command;
   int lcd;
 
+  if (vector_active (vline) == 0)
+    {
+      *status = CMD_ERR_NO_MATCH;
+      return NULL;
+    }
+  else
+    index = vector_active (vline) - 1;
+
   /* First, filter by preceeding command string */
   for (i = 0; i < index; i++)
-    {
-      enum match_type match;
-      int ret;
+    if ((command = vector_slot (vline, i)))
+      {
+	enum match_type match;
+	int ret;
 
-      command = vector_slot (vline, i);
-
-      /* First try completion match, if there is exactly match return 1 */
-      match = cmd_filter_by_completion (command, cmd_vector, i);
-
-      /* If there is exact match then filter ambiguous match else check
-	 ambiguousness. */
-      if ((ret = is_cmd_ambiguous (command, cmd_vector, i, match)) == 1)
-	{
-	  vector_free (cmd_vector);
-	  *status = CMD_ERR_AMBIGUOUS;
-	  return NULL;
-	}
-      /*
-	else if (ret == 2)
-	{
-	  vector_free (cmd_vector);
-	  *status = CMD_ERR_NO_MATCH;
-	  return NULL;
-	}
-      */
-    }
+	/* First try completion match, if there is exactly match return 1 */
+	match = cmd_filter_by_completion (command, cmd_vector, i);
 
+	/* If there is exact match then filter ambiguous match else check
+	   ambiguousness. */
+	if ((ret = is_cmd_ambiguous (command, cmd_vector, i, match)) == 1)
+	  {
+	    vector_free (cmd_vector);
+	    *status = CMD_ERR_AMBIGUOUS;
+	    return NULL;
+	  }
+	/*
+	   else if (ret == 2)
+	   {
+	   vector_free (cmd_vector);
+	   *status = CMD_ERR_NO_MATCH;
+	   return NULL;
+	   }
+	 */
+      }
+  
   /* Prepare match vector. */
   matchvec = vector_init (INIT_MATCHVEC_SIZE);
 
   /* Now we got into completion */
-  for (i = 0; i < vector_max (cmd_vector); i++)
-    if ((cmd_element = vector_slot (cmd_vector, i)) != NULL)
+  for (i = 0; i < vector_active (cmd_vector); i++)
+    if ((cmd_element = vector_slot (cmd_vector, i)))
       {
 	const char *string;
 	vector strvec = cmd_element->strvec;
-	
+
 	/* Check field length */
-	if (index >= vector_max (strvec))
+	if (index >= vector_active (strvec))
 	  vector_slot (cmd_vector, i) = NULL;
-	else 
+	else
 	  {
 	    unsigned int j;
 
 	    descvec = vector_slot (strvec, index);
-	    for (j = 0; j < vector_max (descvec); j++)
-	      {
-		desc = vector_slot (descvec, j);
-
-		if ((string = cmd_entry_function (vector_slot (vline, index),
-						  desc->cmd)))
-		  if (cmd_unique_string (matchvec, string))
-		    vector_set (matchvec, XSTRDUP (MTYPE_TMP, string));
-	      }
+	    for (j = 0; j < vector_active (descvec); j++)
+	      if ((desc = vector_slot (descvec, j)))
+		{
+		  if ((string = 
+		       cmd_entry_function (vector_slot (vline, index),
+					   desc->cmd)))
+		    if (cmd_unique_string (matchvec, string))
+		      vector_set (matchvec, XSTRDUP (MTYPE_TMP, string));
+		}
 	  }
       }
 
@@ -1869,11 +1881,11 @@ cmd_complete_command_real (vector vline, struct vty *vty, int *status)
       if (lcd)
 	{
 	  int len = strlen (vector_slot (vline, index));
-	  
+
 	  if (len < lcd)
 	    {
 	      char *lcdstr;
-	      
+
 	      lcdstr = XMALLOC (MTYPE_TMP, lcd + 1);
 	      memcpy (lcdstr, matchvec->index[0], lcd);
 	      lcdstr[lcd] = '\0';
@@ -1881,14 +1893,14 @@ cmd_complete_command_real (vector vline, struct vty *vty, int *status)
 	      /* match_str = (char **) &lcdstr; */
 
 	      /* Free matchvec. */
-	      for (i = 0; i < vector_max (matchvec); i++)
+	      for (i = 0; i < vector_active (matchvec); i++)
 		{
 		  if (vector_slot (matchvec, i))
 		    XFREE (MTYPE_TMP, vector_slot (matchvec, i));
 		}
 	      vector_free (matchvec);
 
-      	      /* Make new matchvec. */
+	      /* Make new matchvec. */
 	      matchvec = vector_init (INIT_MATCHVEC_SIZE);
 	      vector_set (matchvec, lcdstr);
 	      match_str = (char **) matchvec->index;
@@ -1923,7 +1935,7 @@ cmd_complete_command (vector vline, struct vty *vty, int *status)
 
       shifted_vline = vector_init (vector_count(vline));
       /* use memcpy? */
-      for (index = 1; index < vector_max (vline); index++) 
+      for (index = 1; index < vector_active (vline); index++) 
 	{
 	  vector_set_index (shifted_vline, index-1, vector_lookup(vline, index));
 	}
@@ -1941,7 +1953,7 @@ cmd_complete_command (vector vline, struct vty *vty, int *status)
 
 /* return parent node */
 /* MUST eventually converge on CONFIG_NODE */
-static enum node_type
+enum node_type
 node_parent ( enum node_type node )
 {
   enum node_type ret;
@@ -1968,7 +1980,8 @@ node_parent ( enum node_type node )
 
 /* Execute command by argument vline vector. */
 static int
-cmd_execute_command_real (vector vline, struct vty *vty, struct cmd_element **cmd)
+cmd_execute_command_real (vector vline, struct vty *vty,
+			  struct cmd_element **cmd)
 {
   unsigned int i;
   unsigned int index;
@@ -1985,41 +1998,38 @@ cmd_execute_command_real (vector vline, struct vty *vty, struct cmd_element **cm
   /* Make copy of command elements. */
   cmd_vector = vector_copy (cmd_node_vector (cmdvec, vty->node));
 
-  for (index = 0; index < vector_max (vline); index++) 
-    {
-      int ret;
+  for (index = 0; index < vector_active (vline); index++)
+    if ((command = vector_slot (vline, index)))
+      {
+	int ret;
 
-      command = vector_slot (vline, index);
+	match = cmd_filter_by_completion (command, cmd_vector, index);
 
-      match = cmd_filter_by_completion (command, cmd_vector, index);
+	if (match == vararg_match)
+	  break;
+        
+	ret = is_cmd_ambiguous (command, cmd_vector, index, match);
 
-      if (match == vararg_match)
-	break;
-
-      ret = is_cmd_ambiguous (command, cmd_vector, index, match);
-
-      if (ret == 1)
-	{
-	  vector_free (cmd_vector);
-	  return CMD_ERR_AMBIGUOUS;
-	}
-      else if (ret == 2)
-	{
-	  vector_free (cmd_vector);
-	  return CMD_ERR_NO_MATCH;
-	}
-    }
+	if (ret == 1)
+	  {
+	    vector_free (cmd_vector);
+	    return CMD_ERR_AMBIGUOUS;
+	  }
+	else if (ret == 2)
+	  {
+	    vector_free (cmd_vector);
+	    return CMD_ERR_NO_MATCH;
+	  }
+      }
 
   /* Check matched count. */
   matched_element = NULL;
   matched_count = 0;
   incomplete_count = 0;
 
-  for (i = 0; i < vector_max (cmd_vector); i++) 
-    if (vector_slot (cmd_vector,i) != NULL)
+  for (i = 0; i < vector_active (cmd_vector); i++)
+    if ((cmd_element = vector_slot (cmd_vector, i)))
       {
-	cmd_element = vector_slot (cmd_vector,i);
-
 	if (match == vararg_match || index >= cmd_element->cmdsize)
 	  {
 	    matched_element = cmd_element;
@@ -2033,12 +2043,12 @@ cmd_execute_command_real (vector vline, struct vty *vty, struct cmd_element **cm
 	    incomplete_count++;
 	  }
       }
-  
+
   /* Finish of using cmd_vector. */
   vector_free (cmd_vector);
 
-  /* To execute command, matched_count must be 1.*/
-  if (matched_count == 0) 
+  /* To execute command, matched_count must be 1. */
+  if (matched_count == 0)
     {
       if (incomplete_count)
 	return CMD_ERR_INCOMPLETE;
@@ -2046,22 +2056,22 @@ cmd_execute_command_real (vector vline, struct vty *vty, struct cmd_element **cm
 	return CMD_ERR_NO_MATCH;
     }
 
-  if (matched_count > 1) 
+  if (matched_count > 1)
     return CMD_ERR_AMBIGUOUS;
 
   /* Argument treatment */
   varflag = 0;
   argc = 0;
 
-  for (i = 0; i < vector_max (vline); i++)
+  for (i = 0; i < vector_active (vline); i++)
     {
       if (varflag)
 	argv[argc++] = vector_slot (vline, i);
       else
-	{	  
+	{
 	  vector descvec = vector_slot (matched_element->strvec, i);
 
-	  if (vector_max (descvec) == 1)
+	  if (vector_active (descvec) == 1)
 	    {
 	      struct desc *desc = vector_slot (descvec, 0);
 
@@ -2090,7 +2100,6 @@ cmd_execute_command_real (vector vline, struct vty *vty, struct cmd_element **cm
   return (*matched_element->func) (matched_element, vty, argc, argv);
 }
 
-
 int
 cmd_execute_command (vector vline, struct vty *vty, struct cmd_element **cmd,
 		     int vtysh) {
@@ -2109,7 +2118,7 @@ cmd_execute_command (vector vline, struct vty *vty, struct cmd_element **cmd,
 
       shifted_vline = vector_init (vector_count(vline));
       /* use memcpy? */
-      for (index = 1; index < vector_max (vline); index++) 
+      for (index = 1; index < vector_active (vline); index++) 
 	{
 	  vector_set_index (shifted_vline, index-1, vector_lookup(vline, index));
 	}
@@ -2150,7 +2159,7 @@ cmd_execute_command (vector vline, struct vty *vty, struct cmd_element **cmd,
 
 /* Execute command by argument readline. */
 int
-cmd_execute_command_strict (vector vline, struct vty *vty, 
+cmd_execute_command_strict (vector vline, struct vty *vty,
 			    struct cmd_element **cmd)
 {
   unsigned int i;
@@ -2168,40 +2177,39 @@ cmd_execute_command_strict (vector vline, struct vty *vty,
   /* Make copy of command element */
   cmd_vector = vector_copy (cmd_node_vector (cmdvec, vty->node));
 
-  for (index = 0; index < vector_max (vline); index++) 
-    {
-      int ret;
+  for (index = 0; index < vector_active (vline); index++)
+    if ((command = vector_slot (vline, index)))
+      {
+	int ret;
+	
+	match = cmd_filter_by_string (vector_slot (vline, index),
+				      cmd_vector, index);
 
-      command = vector_slot (vline, index);
-
-      match = cmd_filter_by_string (vector_slot (vline, index), 
-				    cmd_vector, index);
-
-      /* If command meets '.VARARG' then finish matching. */
-      if (match == vararg_match)
-	break;
-
-      ret = is_cmd_ambiguous (command, cmd_vector, index, match);
-      if (ret == 1)
-	{
-	  vector_free (cmd_vector);
-	  return CMD_ERR_AMBIGUOUS;
-	}
-      if (ret == 2)
-	{
-	  vector_free (cmd_vector);
-	  return CMD_ERR_NO_MATCH;
-	}
-    }
+	/* If command meets '.VARARG' then finish matching. */
+	if (match == vararg_match)
+	  break;
+        
+	ret = is_cmd_ambiguous (command, cmd_vector, index, match);
+	if (ret == 1)
+	  {
+	    vector_free (cmd_vector);
+	    return CMD_ERR_AMBIGUOUS;
+	  }
+	if (ret == 2)
+	  {
+	    vector_free (cmd_vector);
+	    return CMD_ERR_NO_MATCH;
+	  }
+      }
 
   /* Check matched count. */
   matched_element = NULL;
   matched_count = 0;
   incomplete_count = 0;
-  for (i = 0; i < vector_max (cmd_vector); i++) 
-    if (vector_slot (cmd_vector,i) != NULL)
+  for (i = 0; i < vector_active (cmd_vector); i++)
+    if (vector_slot (cmd_vector, i) != NULL)
       {
-	cmd_element = vector_slot (cmd_vector,i);
+	cmd_element = vector_slot (cmd_vector, i);
 
 	if (match == vararg_match || index >= cmd_element->cmdsize)
 	  {
@@ -2211,12 +2219,12 @@ cmd_execute_command_strict (vector vline, struct vty *vty,
 	else
 	  incomplete_count++;
       }
-  
+
   /* Finish of using cmd_vector. */
   vector_free (cmd_vector);
 
-  /* To execute command, matched_count must be 1.*/
-  if (matched_count == 0) 
+  /* To execute command, matched_count must be 1. */
+  if (matched_count == 0)
     {
       if (incomplete_count)
 	return CMD_ERR_INCOMPLETE;
@@ -2224,28 +2232,28 @@ cmd_execute_command_strict (vector vline, struct vty *vty,
 	return CMD_ERR_NO_MATCH;
     }
 
-  if (matched_count > 1) 
+  if (matched_count > 1)
     return CMD_ERR_AMBIGUOUS;
 
   /* Argument treatment */
   varflag = 0;
   argc = 0;
 
-  for (i = 0; i < vector_max (vline); i++)
+  for (i = 0; i < vector_active (vline); i++)
     {
       if (varflag)
 	argv[argc++] = vector_slot (vline, i);
       else
-	{	  
+	{
 	  vector descvec = vector_slot (matched_element->strvec, i);
 
-	  if (vector_max (descvec) == 1)
+	  if (vector_active (descvec) == 1)
 	    {
 	      struct desc *desc = vector_slot (descvec, 0);
 
 	      if (CMD_VARARG (desc->cmd))
 		varflag = 1;
-	  
+
 	      if (varflag || CMD_VARIABLE (desc->cmd) || CMD_OPTION (desc->cmd))
 		argv[argc++] = vector_slot (vline, i);
 	    }
@@ -2487,8 +2495,10 @@ DEFUN (config_list,
   struct cmd_node *cnode = vector_slot (cmdvec, vty->node);
   struct cmd_element *cmd;
 
-  for (i = 0; i < vector_max (cnode->cmd_vector); i++)
-    if ((cmd = vector_slot (cnode->cmd_vector, i)) != NULL)
+  for (i = 0; i < vector_active (cnode->cmd_vector); i++)
+    if ((cmd = vector_slot (cnode->cmd_vector, i)) != NULL
+        && !(cmd->attr == CMD_ATTR_DEPRECATED
+             || cmd->attr == CMD_ATTR_HIDDEN))
       vty_out (vty, "  %s%s", cmd->string,
 	       VTY_NEWLINE);
   return CMD_SUCCESS;
@@ -2549,7 +2559,7 @@ DEFUN (config_write_file,
   vty_time_print (file_vty, 1);
   vty_out (file_vty, "!\n");
 
-  for (i = 0; i < vector_max (cmdvec); i++)
+  for (i = 0; i < vector_active (cmdvec); i++)
     if ((node = vector_slot (cmdvec, i)) && node->func)
       {
 	if ((*node->func) (file_vty))
@@ -2643,7 +2653,7 @@ DEFUN (config_write_terminal,
 
   if (vty->type == VTY_SHELL_SERV)
     {
-      for (i = 0; i < vector_max (cmdvec); i++)
+      for (i = 0; i < vector_active (cmdvec); i++)
 	if ((node = vector_slot (cmdvec, i)) && node->func && node->vtysh)
 	  {
 	    if ((*node->func) (vty))
@@ -2656,7 +2666,7 @@ DEFUN (config_write_terminal,
 	       VTY_NEWLINE);
       vty_out (vty, "!%s", VTY_NEWLINE);
 
-      for (i = 0; i < vector_max (cmdvec); i++)
+      for (i = 0; i < vector_active (cmdvec); i++)
 	if ((node = vector_slot (cmdvec, i)) && node->func)
 	  {
 	    if ((*node->func) (vty))
@@ -3009,8 +3019,10 @@ DEFUN_HIDDEN (do_echo,
 {
   char *message;
 
-  vty_out (vty, "%s%s",(message = argv_concat(argv, argc, 0)), VTY_NEWLINE);
-  XFREE(MTYPE_TMP, message);
+  vty_out (vty, "%s%s", ((message = argv_concat(argv, argc, 0)) ? message : ""),
+	   VTY_NEWLINE);
+  if (message)
+    XFREE(MTYPE_TMP, message);
   return CMD_SUCCESS;
 }
 
@@ -3027,8 +3039,9 @@ DEFUN (config_logmsg,
   if ((level = level_match(argv[0])) == ZLOG_DISABLED)
     return CMD_ERR_NO_MATCH;
 
-  zlog(NULL, level, (message = argv_concat(argv, argc, 1)));
-  XFREE(MTYPE_TMP, message);
+  zlog(NULL, level, ((message = argv_concat(argv, argc, 1)) ? message : ""));
+  if (message)
+    XFREE(MTYPE_TMP, message);
   return CMD_SUCCESS;
 }
 
@@ -3403,6 +3416,20 @@ DEFUN (no_config_log_record_priority,
   return CMD_SUCCESS;
 }
 
+DEFUN (banner_motd_file,
+       banner_motd_file_cmd,
+       "banner motd file [FILE]",
+       "Set banner\n"
+       "Banner for motd\n"
+       "Banner from a file\n"
+       "Filename\n")
+{
+  if (host.motdfile)
+    XFREE (MTYPE_TMP, host.motdfile);
+  host.motdfile = XSTRDUP (MTYPE_TMP, argv[0]);
+
+  return CMD_SUCCESS;
+}
 
 DEFUN (banner_motd_default,
        banner_motd_default_cmd,
@@ -3423,6 +3450,9 @@ DEFUN (no_banner_motd,
        "Strings for motd\n")
 {
   host.motd = NULL;
+  if (host.motdfile) 
+    XFREE (MTYPE_TMP, host.motdfile);
+  host.motdfile = NULL;
   return CMD_SUCCESS;
 }
 
@@ -3464,6 +3494,7 @@ cmd_init (int terminal)
   host.config = NULL;
   host.lines = -1;
   host.motd = default_motd;
+  host.motdfile = NULL;
 
   /* Install top nodes. */
   install_node (&view_node, NULL);
@@ -3543,12 +3574,15 @@ cmd_init (int terminal)
       install_element (CONFIG_NODE, &service_password_encrypt_cmd);
       install_element (CONFIG_NODE, &no_service_password_encrypt_cmd);
       install_element (CONFIG_NODE, &banner_motd_default_cmd);
+      install_element (CONFIG_NODE, &banner_motd_file_cmd);
       install_element (CONFIG_NODE, &no_banner_motd_cmd);
       install_element (CONFIG_NODE, &service_terminal_length_cmd);
       install_element (CONFIG_NODE, &no_service_terminal_length_cmd);
 
-      install_element(VIEW_NODE, &show_thread_cpu_cmd);
-      install_element(ENABLE_NODE, &show_thread_cpu_cmd);
+      install_element (VIEW_NODE, &show_thread_cpu_cmd);
+      install_element (ENABLE_NODE, &show_thread_cpu_cmd);
+      install_element (VIEW_NODE, &show_work_queues_cmd);
+      install_element (ENABLE_NODE, &show_work_queues_cmd);
     }
   srand(time(NULL));
 }
