@@ -26,7 +26,6 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "prefix.h"
 #include "routemap.h"
 #include "command.h"
-#include "vty.h"
 #include "log.h"
 
 /* Vector for route match rules. */
@@ -210,6 +209,11 @@ vty_show_route_map_entry (struct vty *vty, struct route_map *map)
       vty_out (vty, "route-map %s, %s, sequence %d%s",
                map->name, route_map_type_str (index->type),
                index->pref, VTY_NEWLINE);
+
+      /* Description */
+      if (index->description)
+	vty_out (vty, "  Description:%s    %s%s", VTY_NEWLINE,
+		 index->description, VTY_NEWLINE);
       
       /* Match clauses */
       vty_out (vty, "  Match clauses:%s", VTY_NEWLINE);
@@ -222,17 +226,20 @@ vty_show_route_map_entry (struct vty *vty, struct route_map *map)
         vty_out (vty, "    %s %s%s",
                  rule->cmd->str, rule->rule_str, VTY_NEWLINE);
       
-      /* Call clause */
-      vty_out (vty, "  Call clause:%s", VTY_NEWLINE);
+      vty_out (vty, "  Action:%s", VTY_NEWLINE);
+      
       if (index->nextrm)
         vty_out (vty, "    Call %s%s", index->nextrm, VTY_NEWLINE);
-      
-      /* Exit Policy */
-      vty_out (vty, "  Action:%s", VTY_NEWLINE);
-      if (index->exitpolicy == RMAP_GOTO)
+      else if (index->exitpolicy == RMAP_GOTO)
         vty_out (vty, "    Goto %d%s", index->nextpref, VTY_NEWLINE);
       else if (index->exitpolicy == RMAP_NEXT)
-        vty_out (vty, "    Continue to next entry%s", VTY_NEWLINE);
+        {
+          vty_out (vty, "    Goto next, (entry ");
+          if (index->next)
+            vty_out (vty, " %d)%s", index->next->pref, VTY_NEWLINE);
+          else
+            vty_out (vty, " undefined)%s", VTY_NEWLINE);
+        }
       else if (index->exitpolicy == RMAP_EXIT)
         vty_out (vty, "    Exit routemap%s", VTY_NEWLINE);
     }
@@ -427,7 +434,7 @@ route_map_lookup_match (const char *name)
   unsigned int i;
   struct route_map_rule_cmd *rule;
 
-  for (i = 0; i < vector_max (route_match_vec); i++)
+  for (i = 0; i < vector_active (route_match_vec); i++)
     if ((rule = vector_slot (route_match_vec, i)) != NULL)
       if (strcmp (rule->str, name) == 0)
 	return rule;
@@ -441,7 +448,7 @@ route_map_lookup_set (const char *name)
   unsigned int i;
   struct route_map_rule_cmd *rule;
 
-  for (i = 0; i < vector_max (route_set_vec); i++)
+  for (i = 0; i < vector_active (route_set_vec); i++)
     if ((rule = vector_slot (route_set_vec, i)) != NULL)
       if (strcmp (rule->str, name) == 0)
 	return rule;
@@ -1056,16 +1063,15 @@ DEFUN (rmap_onmatch_goto,
        "Goto Clause number\n"
        "Number\n")
 {
-  struct route_map_index *index = vty->index;
+  struct route_map_index *index;
   int d = 0;
 
+  if (argv[0])
+    d = atoi(argv[0]);
+
+  index = vty->index;
   if (index)
     {
-      if (argc == 1 && argv[0])
-        VTY_GET_INTEGER_RANGE("route-map index", d, argv[0], 1, 65536)
-      else
-        d = index->pref + 1;
-      
       if (d <= index->pref)
 	{
 	  /* Can't allow you to do that, Dave */
@@ -1187,6 +1193,42 @@ DEFUN (no_rmap_call,
   return CMD_SUCCESS;
 }
 
+DEFUN (rmap_description,
+       rmap_description_cmd,
+       "description .LINE",
+       "Route-map comment\n"
+       "Comment describing this route-map rule\n")
+{
+  struct route_map_index *index;
+
+  index = vty->index;
+  if (index)
+    {
+      if (index->description)
+	XFREE (MTYPE_TMP, index->description);
+      index->description = argv_concat (argv, argc, 0);
+    }
+  return CMD_SUCCESS;
+}
+
+DEFUN (no_rmap_description,
+       no_rmap_description_cmd,
+       "no description",
+       NO_STR
+       "Route-map comment\n")
+{
+  struct route_map_index *index;
+
+  index = vty->index;
+  if (index)
+    {
+      if (index->description)
+	XFREE (MTYPE_TMP, index->description);
+      index->description = NULL;
+    }
+  return CMD_SUCCESS;
+}
+
 /* Configuration write function. */
 int
 route_map_config_write (struct vty *vty)
@@ -1210,6 +1252,9 @@ route_map_config_write (struct vty *vty)
 		 route_map_type_str (index->type),
 		 index->pref, VTY_NEWLINE);
 
+	if (index->description)
+	  vty_out (vty, " description %s%s", index->description, VTY_NEWLINE);
+
 	for (rule = index->match_list.head; rule; rule = rule->next)
 	  vty_out (vty, " match %s %s%s", rule->cmd->str, 
 		   rule->rule_str ? rule->rule_str : "",
@@ -1267,6 +1312,10 @@ route_map_init_vty ()
   /* Install the call stuff. */
   install_element (RMAP_NODE, &rmap_call_cmd);
   install_element (RMAP_NODE, &no_rmap_call_cmd);
+
+  /* Install description commands. */
+  install_element (RMAP_NODE, &rmap_description_cmd);
+  install_element (RMAP_NODE, &no_rmap_description_cmd);
    
   /* Install show command */
   install_element (ENABLE_NODE, &rmap_show_cmd);
