@@ -61,12 +61,10 @@ ospf6_neighbor_lookup (u_int32_t router_id,
   struct listnode *n;
   struct ospf6_neighbor *on;
 
-  for (n = listhead (oi->neighbor_list); n; nextnode (n))
-    {
-      on = (struct ospf6_neighbor *) getdata (n);
-      if (on->router_id == router_id)
-        return on;
-    }
+  for (ALL_LIST_ELEMENTS_RO (oi->neighbor_list, n, on))
+    if (on->router_id == router_id)
+      return on;
+  
   return (struct ospf6_neighbor *) NULL;
 }
 
@@ -161,9 +159,9 @@ ospf6_neighbor_state_change (u_char next_state, struct ospf6_neighbor *on)
   /* log */
   if (IS_OSPF6_DEBUG_NEIGHBOR (STATE))
     {
-      zlog_info ("Neighbor state change %s: [%s]->[%s]", on->name,
-                 ospf6_neighbor_state_str[prev_state],
-                 ospf6_neighbor_state_str[next_state]);
+      zlog_debug ("Neighbor state change %s: [%s]->[%s]", on->name,
+		  ospf6_neighbor_state_str[prev_state],
+		  ospf6_neighbor_state_str[next_state]);
     }
 
   if (prev_state == OSPF6_NEIGHBOR_FULL || next_state == OSPF6_NEIGHBOR_FULL)
@@ -218,7 +216,7 @@ hello_received (struct thread *thread)
   assert (on);
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *HelloReceived*", on->name);
+    zlog_debug ("Neighbor Event %s: *HelloReceived*", on->name);
 
   /* reset Inactivity Timer */
   THREAD_OFF (on->inactivity_timer);
@@ -243,7 +241,7 @@ twoway_received (struct thread *thread)
     return 0;
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *2Way-Received*", on->name);
+    zlog_debug ("Neighbor Event %s: *2Way-Received*", on->name);
 
   thread_add_event (master, neighbor_change, on->ospf6_if, 0);
 
@@ -278,7 +276,7 @@ negotiation_done (struct thread *thread)
     return 0;
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *NegotiationDone*", on->name);
+    zlog_debug ("Neighbor Event %s: *NegotiationDone*", on->name);
 
   /* clear ls-list */
   ospf6_lsdb_remove_all (on->summary_list);
@@ -347,7 +345,7 @@ exchange_done (struct thread *thread)
     return 0;
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *ExchangeDone*", on->name);
+    zlog_debug ("Neighbor Event %s: *ExchangeDone*", on->name);
 
   THREAD_OFF (on->thread_send_dbdesc);
   ospf6_lsdb_remove_all (on->dbdesc_list);
@@ -377,7 +375,7 @@ loading_done (struct thread *thread)
     return 0;
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *LoadingDone*", on->name);
+    zlog_debug ("Neighbor Event %s: *LoadingDone*", on->name);
 
   assert (on->request_list->count == 0);
 
@@ -396,7 +394,7 @@ adj_ok (struct thread *thread)
   assert (on);
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *AdjOK?*", on->name);
+    zlog_debug ("Neighbor Event %s: *AdjOK?*", on->name);
 
   if (on->state == OSPF6_NEIGHBOR_TWOWAY && need_adjacency (on))
     {
@@ -440,7 +438,7 @@ seqnumber_mismatch (struct thread *thread)
     return 0;
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *SeqNumberMismatch*", on->name);
+    zlog_debug ("Neighbor Event %s: *SeqNumberMismatch*", on->name);
 
   ospf6_neighbor_state_change (OSPF6_NEIGHBOR_EXSTART, on);
   SET_FLAG (on->dbdesc_bits, OSPF6_DBDESC_MSBIT);
@@ -476,7 +474,7 @@ bad_lsreq (struct thread *thread)
     return 0;
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *BadLSReq*", on->name);
+    zlog_debug ("Neighbor Event %s: *BadLSReq*", on->name);
 
   ospf6_neighbor_state_change (OSPF6_NEIGHBOR_EXSTART, on);
   SET_FLAG (on->dbdesc_bits, OSPF6_DBDESC_MSBIT);
@@ -512,7 +510,7 @@ oneway_received (struct thread *thread)
     return 0;
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *1Way-Received*", on->name);
+    zlog_debug ("Neighbor Event %s: *1Way-Received*", on->name);
 
   ospf6_neighbor_state_change (OSPF6_NEIGHBOR_INIT, on);
   thread_add_event (master, neighbor_change, on->ospf6_if, 0);
@@ -543,7 +541,7 @@ inactivity_timer (struct thread *thread)
   assert (on);
 
   if (IS_OSPF6_DEBUG_NEIGHBOR (EVENT))
-    zlog_info ("Neighbor Event %s: *InactivityTimer*", on->name);
+    zlog_debug ("Neighbor Event %s: *InactivityTimer*", on->name);
 
   on->inactivity_timer = NULL;
   on->drouter = on->prev_drouter = 0;
@@ -795,19 +793,11 @@ DEFUN (show_ipv6_ospf6_neighbor,
              "RouterID", "State", "Duration", "DR", "BDR", "I/F",
              "State", VNL);
 
-  for (i = listhead (ospf6->area_list); i; nextnode (i))
-    {
-      oa = (struct ospf6_area *) getdata (i);
-      for (j = listhead (oa->if_list); j; nextnode (j))
-        {
-          oi = (struct ospf6_interface *) getdata (j);
-          for (k = listhead (oi->neighbor_list); k; nextnode (k))
-            {
-              on = (struct ospf6_neighbor *) getdata (k);
-              (*showfunc) (vty, on);
-            }
-        }
-    }
+  for (ALL_LIST_ELEMENTS_RO (ospf6->area_list, i, oa))
+    for (ALL_LIST_ELEMENTS_RO (oa->if_list, j, oi))
+      for (ALL_LIST_ELEMENTS_RO (oi->neighbor_list, k, on))
+        (*showfunc) (vty, on);
+
   return CMD_SUCCESS;
 }
 
@@ -849,20 +839,11 @@ DEFUN (show_ipv6_ospf6_neighbor_one,
       return CMD_SUCCESS;
     }
 
-  for (i = listhead (ospf6->area_list); i; nextnode (i))
-    {
-      oa = (struct ospf6_area *) getdata (i);
-      for (j = listhead (oa->if_list); j; nextnode (j))
-        {
-          oi = (struct ospf6_interface *) getdata (j);
-          for (k = listhead (oi->neighbor_list); k; nextnode (k))
-            {
-              on = (struct ospf6_neighbor *) getdata (k);
-              if (on->router_id == router_id)
-                (*showfunc) (vty, on);
-            }
-        }
-    }
+  for (ALL_LIST_ELEMENTS_RO (ospf6->area_list, i, oa))
+    for (ALL_LIST_ELEMENTS_RO (oa->if_list, j, oi))
+      for (ALL_LIST_ELEMENTS_RO (oi->neighbor_list, k, on))
+        (*showfunc) (vty, on);
+  
   return CMD_SUCCESS;
 }
 
