@@ -53,10 +53,8 @@ ospf_dr_election_sub (struct list *routers)
 
   /* Choose highest router priority.
      In case of tie, choose highest Router ID. */
-  for (node = listhead (routers); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (routers, node, nbr))
     {
-      nbr = getdata (node);
-
       if (max == NULL)
 	max = nbr;
       else
@@ -82,10 +80,8 @@ ospf_elect_dr (struct ospf_interface *oi, struct list *el_list)
   dr_list = list_new ();
 
   /* Add neighbors to the list. */
-  for (node = listhead (el_list); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (el_list, node, nbr))
     {
-      nbr = getdata (node);
-
       /* neighbor declared to be DR. */
       if (NBR_IS_DR (nbr))
 	listnode_add (dr_list, nbr);
@@ -126,10 +122,8 @@ ospf_elect_bdr (struct ospf_interface *oi, struct list *el_list)
   no_dr_list = list_new ();
 
   /* Add neighbors to the list. */
-  for (node = listhead (el_list); node; nextnode (node))
+  for (ALL_LIST_ELEMENTS_RO (el_list, node, nbr))
     {
-      nbr = getdata (node);
-
       /* neighbor declared to be DR. */
       if (NBR_IS_DR (nbr))
 	continue;
@@ -254,17 +248,6 @@ ospf_dr_election (struct ospf_interface *oi)
       !IPV4_ADDR_SAME (&old_bdr, &BDR (oi)))
     ospf_dr_change (oi->ospf, oi->nbrs);
 
-  if (oi->type == OSPF_IFTYPE_BROADCAST || oi->type == OSPF_IFTYPE_POINTOPOINT)
-    {
-      /* Multicast group change. */
-      if ((old_state != ISM_DR && old_state != ISM_Backup) &&
-	  (new_state == ISM_DR || new_state == ISM_Backup))
-	ospf_if_add_alldrouters (oi->ospf, oi->address, oi->ifp->ifindex);
-      else if ((old_state == ISM_DR || old_state == ISM_Backup) &&
-	       (new_state != ISM_DR && new_state != ISM_Backup))
-	ospf_if_drop_alldrouters (oi->ospf, oi->address, oi->ifp->ifindex);
-    }
-
   return new_state;
 }
 
@@ -581,6 +564,9 @@ ism_change_state (struct ospf_interface *oi, int state)
   oi->state = state;
   oi->state_change++;
 
+  /* Set multicast memberships appropriately for new state. */
+  ospf_if_set_multicast(oi);
+
   if (old_state == ISM_Down || state == ISM_Down)
     ospf_check_abr_status (oi->ospf);
 
