@@ -32,6 +32,9 @@ struct rib
   struct rib *next;
   struct rib *prev;
 
+  /* ref count */
+  unsigned int lock;
+  
   /* Type fo this route. */
   int type;
 
@@ -244,8 +247,7 @@ static_delete_ipv4 (struct prefix *p, struct in_addr *gate, const char *ifname,
 #ifdef HAVE_IPV6
 int
 rib_add_ipv6 (int type, int flags, struct prefix_ipv6 *p,
-	      struct in6_addr *gate, unsigned int ifindex, u_int32_t vrf_id,
-	      u_int32_t metric, u_char distance);
+	      struct in6_addr *gate, unsigned int ifindex, u_int32_t vrf_id);
 
 int
 rib_delete_ipv6 (int type, int flags, struct prefix_ipv6 *p,
